           (*-------------------------------------------*
            |                CSP-Prover                 |
            |               December 2004               |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory CSP_Prover = CSP_tactic:

(*****************************************************************

         1. CSP-Prover
         2.
         3. 
         4. 

 *****************************************************************)

(*-------------------------------------------------------*
 |                                                       |
 |                 default LET _ In _                    |
 |                                                       |
 *-------------------------------------------------------*)

syntax
  "_LetinUFP" :: "('n,'a) procDF => ('n,'a) proc => ('n,'a) procRC"
                                    ("//(2LET /_ )/(2IN /_)/" [53,53] 53)

translations
  "LET df IN P"     == "LET:Ufp df IN P"         (* default LET _ IN _ *)

(*-------------------------------------------------------*
 |                                                       |
 |      adding ... to automatic simplification rules     |
 |                                                       |
 *-------------------------------------------------------*)

(* automatically unfolded functions *)

declare Ext_choice_step_def                        [simp]
declare Parallel_step_def                          [simp]
declare Hiding_step_def                            [simp]
declare Renaming_step_def                          [simp]
declare Seq_compo_step_def                         [simp]

(*-------------------------------------------------------------*
 |                                                             |
 |  Users may be sometimes required to apply "Int_prefix_def"  |
 |  for unfoling "! x:X -> P". Do you like the following ?     |
 |                                                             |
 |  declare Int_pre_choice_def                     [simp]      |
 |                                                             |
 |  Users may be sometimes required to apply "Send_prefix_def" |
 |  for unfoling "a ! x: -> P". Do you like the following ?    |
 |                                                             |
 |  declare Send_prefix_def                        [simp]      |
 |                                                             |
 |  Users may be sometimes required to apply "Rec_prefix_def"  |
 |  for unfoling "a ? x:X -> P". Do you like the following ?   |
 |                                                             |
 |  declare Rec_prefix_def                         [simp]      |
 |                                                             |
 *-------------------------------------------------------------*)

(*                           NO                                *)

(*----------------------------------------------------------------------*
 |                                                                      |
 |  To unfold (resp. fold) syntactic-sugar for Ext_ and Int_pre_choices |
 |  choices, use "unfold csp_ss_def" (resp. "fold csp_ss_def").         |
 |                                                                      |
 *----------------------------------------------------------------------*)

(*---------------------------------------------------------------------*
 | Nondet_send_prefix_def : non-deterministic sending                  |
 | Rep_int_choice_fun_def : Replicated internal choice with a function |
 *---------------------------------------------------------------------*)

(*   "inj_on_def" is needed for resolving (inv f) in R_Prefix_def *)

declare inj_on_def                                 [simp]

(*------------------------------------*
 |                                    |
 |    laws automatically applied      |
 |                                    |
 *------------------------------------*)

(* intro! intro? are automatically applied by "rule".     *)
(* intro! is automatically applied by "rules" and "auto". *)

(* CSP_SF_law_commut *)

declare csp_commut                                  [simp]

(* CSP_SF_law_ref *)

declare csp_Int_choice_right                        [intro!]
declare csp_Rep_int_choice_right                    [intro!]

(* 
declare csp_Int_choice_left1                        [intro?]
declare csp_Int_choice_left2                        [intro?]
declare csp_Int_choice_left3                        [intro?]
declare csp_Rep_int_choice_left                     [intro?]
declare csp_Rep_int_choice_left_x                   [intro?]

declare csp_decompo_subset                          [intro?]
declare csp_Ext_choice_right                        [intro?]
declare csp_Int_choice_Ext_choice                   [intro?]
*)

(* CSP_SF_law_SKIP *)

declare csp_SKIP                                    [simp]
lemmas  csp_SKIP_sym                                [simp]
      = csp_SKIP[THEN csp_sym]

(*
declare csp_SKIP_rw                                 [simp]
*)

(* CSP_SF_law_decompo *)

declare csp_rm_head                                 [intro!]
declare csp_decompo                                 [simp]

(*
declare csp_free_decompo                            [intro!] 
declare csp_decompo                                 [intro!] 
declare csp_free_decompo_left                       [intro!] 
declare csp_free_decompo_right                      [intro!] 
declare csp_decompo_left                            [intro!] 
declare csp_decompo_right                           [intro!] 

declare csp_free_decompo_flag_left                  [intro!] 
declare csp_free_decompo_flag_right                 [intro!] 
declare csp_decompo_flag_left                       [intro!] 
declare csp_decompo_flag_right                      [intro!] 
*)

(* CSP_SF_law_dist *)

declare csp_dist                                    [simp]
lemmas  csp_dist_sym                                [simp]
      = csp_dist[THEN csp_sym]

declare csp_Rep_dist                                [simp]
lemmas  csp_Rep_dist_sym                            [simp]
      = csp_Rep_dist[THEN csp_sym]

(*

declare csp_dist_rw                                 [simp]
declare csp_dist_rw_left                            [simp]
declare csp_dist_rw_right                           [simp]
declare csp_Rep_dist_rw                             [simp]
declare csp_Rep_dist_rw_left                        [simp]
declare csp_Rep_dist_rw_right                       [simp]
*)

(* CSP_SF_law_fp *)

declare csp_check                                   [intro!]
lemmas csp_fp_induct = csp_fp_induct_cms
declare csp_unwind                                  [simp]
lemmas  csp_unwind_sym                              [simp]
      = csp_unwind[THEN csp_sym]

(*
declare csp_unwind_rw                               [simp]
declare csp_fp_induct_cms                           [intro!]
*)

(* CSP_SF_law_step *)

declare csp_step                                    [simp]
lemmas  csp_step_sym                                [simp]
      = csp_step[THEN csp_sym]
(*

declare csp_step_rw                                 [simp]
declare csp_step_rw_left                            [simp]
declare csp_step_rw_right                           [simp]

declare light_csp_step_rw                           [simp]
declare light_csp_step_rw_left                      [simp]
declare light_csp_step_rw_right                     [simp]
*)

(* CSP_SF_law_etc *)

declare csp_Conditional                             [simp]

declare csp_simp                                    [simp]
lemmas  csp_simp_sym                                [simp]
      = csp_simp[THEN csp_sym]

(*
declare csp_Conditional_split                       [simp]
declare csp_Conditional_split_rw                    [simp]
declare csp_Conditional_rw                          [simp]

declare csp_reflex                                  [intro!]
declare csp_reflex_ex                               [intro!]

declare csp_simp                                    [simp]
lemmas  csp_simp_sym                                [simp]
      = csp_simp[THEN csp_sym]

declare csp_eq_ref                                  [intro?]
declare csp_ref_eq                                  [intro?]
declare csp_trans                                   [intro?]

declare csp_sym                                     [intro!]
declare csp_symE                                    [elim!]
*)

(*
declare csp_Int_simp_rw                             [simp]
declare csp_Ext_simp_rw                             [simp]
declare csp_Mix_simp_rw                             [simp]
declare csp_simp_rw                                 [simp]
declare csp_Timeout_right                           [intro!]
*)

(***************************************************************
                       Domain theory
 ***************************************************************)

declare memT_IntT                                   [simp]
declare memT_UnT                                    [simp]
declare memF_IntF                                   [simp]
declare memF_UnF                                    [simp]


end
